import os
import win32com.client as win32
import pathlib


class Measurement:
  r""" Open measurement file or bind active measurement

  Parameters
  ----------
  filename : str
             measurement file path

  """
  def __init__(self, filename: str = None):
    self.server  = win32.gencache.EnsureDispatch('NSI2000.server')
    self.app     = self.server.AppConnection
    self.console = self.app.ScriptCommands

    if filename:
      if os.path.exists(filename):
        if pathlib.Path(filename).suffix == ".NSI" or pathlib.Path(filename).suffix == ".nsi":
          try:
            self.console.DATA_FILENAME = filename
            self.file = filename 
          except:
            raise Exception
        else:
          raise ValueError("Filename must be valid .NSI type")
      else:
        raise FileNotFoundError
    else:
      self.file = self.console.DATA_FILENAME

  def MEAS_FREQUENCY(self) -> float:
    r""" Returns beam frequency

    Returns
    -------
    float
      beam freqeuncy in Hertz [Hz]

    """
    return self.console.MEAS_FREQUENCY*1E9

  def CLEAR_ALL_PLOTS(self) -> None:
    r""" Clear all plot windows
    """
    return self.console.CLEAR_ALL_PLOTS()

  def nsi2000Version(self) -> str:
    r""" Returns NSI2000 software version

    Returns
    -------
    str
      version string
    """
    return self.app.Version






import Measurement

class NearField(Measurement):
  r""" Near field processing for selected measurement

  Parameters
  ----------
  meas : Measurement
         selected measurement

  """
  def __init__(self, meas: Measurement):
    self.meas = meas

  def LISTING_TO_FILE(self):
    r""" Writes the far-field data to the specified file for both polarizations 
    """
    filename = self.meas.file.replace(".NSI","_Beam%03d_NF.txt"%(self.meas.console.BEAM_NUMBER+1))
    self.meas.console.NF_LISTING_TO_FILE(filename, True)



import Measurement

import os

class BeamTable(Measurement):
  r""" Beam (Frequency) table operations

  Parameters
  ----------
  meas : Measurement
         selected measurement

  """
  def __init__(self, meas: Measurement):
    self.meas = meas

  def Count(self):
    r""" Returns the number of beams in the beam table.

    Returns
    ----------
    int
      number of beams (frequencies) in the beam table

    """
    self.meas.console.RunScriptFile("C:\\NSI2000\\Script\\ExportBeamCount.bas")
    beamCount = 0
    with open(self.meas.file.replace(".NSI","_BeamCount.txt")) as file:
      beamCount = int(file.readlines()[0])
    os.remove(self.meas.file.replace(".NSI","_BeamCount.txt"))
    return beamCount


  def SELECT_BEAM(self, beam: int) -> None:
    r""" Select beam from beam table

    Parameters
    ----------
    beam : int
          beam number
    """
    self.meas.console.SELECT_BEAM(beam) 

